﻿using System;
using System.Collections.ObjectModel;

namespace InfiniTec.Threading
{
	internal class ResourceReservationCollection : KeyedCollection<IResourceConstraint, ResourceReservation>
	{
		private readonly object _InstanceLock = new object();

		protected override IResourceConstraint GetKeyForItem(ResourceReservation item)
		{
			return item.Constraint;
		}

		public void RegisterOperation(AsyncOperation operation)
		{
			if (operation == null) throw new ArgumentNullException("operation");

			if (operation.ResourceConstraint == null || operation.ResourceConstraint is UnrestrictedConstraint)
			{
				AsyncOperation.StartAsync(operation);
				return;
			}

			lock (_InstanceLock)
			{
				IResourceConstraint key = operation.ResourceConstraint;
				ResourceReservation reservation;
				if (!Contains(key))
				{
					reservation = new ResourceReservation(key);
					Add(reservation);
				}
				else
				{
					reservation = this[key];
				}
				reservation.RegisterOperation(operation);
			}
		}

		public void ReleaseOperation(AsyncOperation operation)
		{
			if (operation == null) throw new ArgumentNullException("operation");

			if (operation.ResourceConstraint == null || operation.ResourceConstraint is UnrestrictedConstraint)
			{
				return;
			}

			lock (_InstanceLock)
			{
				IResourceConstraint key = operation.ResourceConstraint;
				if (!Contains(key))
				{
					throw new InvalidOperationException(string.Format("The resource constraint {0} is not registered", key.GetType()));
				}
				ResourceReservation reservation = this[key];
				reservation.ReleaseOperation(operation);
				if (reservation.Count == 0)
				{
					Remove(key);
				}
			}
		}
	}
}