﻿using System;
using System.ComponentModel;
using System.IO;
using ADODB;

namespace InfiniTec.Exchange.Eventing
{
    public class Connection: IDisposable
    {
        public _Connection _Connection { get; private set; }
        private readonly bool _ConnectionMustBeDisposed;
        private readonly string _Url;

        public Connection(_Connection connection, bool ownsConnection)
        {
            _Connection = connection;
            _ConnectionMustBeDisposed = ownsConnection;
        }

        public Record OpenItem(string url)
        {
            return OpenItem(url, FileAccess.ReadWrite);
        }

        public Record OpenItem(string url, FileAccess access)
        {
            if (_Connection == null)
            {
                OpenConnection();
            }

            var record = new Record();

            ConnectModeEnum accessMode;
            switch (access)
            {
                case FileAccess.Read:
                    accessMode = ConnectModeEnum.adModeRead;
                    break;
                case FileAccess.ReadWrite:
                    accessMode = ConnectModeEnum.adModeReadWrite;
                    break;
                case FileAccess.Write:
                    accessMode = ConnectModeEnum.adModeWrite;
                    break;
                default:
                    throw new InvalidEnumArgumentException("access", (int) access, typeof (FileAccess));
            }

            record.Open(url, _Connection, accessMode, RecordCreateOptionsEnum.adFailIfNotExists, RecordOpenOptionsEnum.adOpenSource, null, null);

            return record;
        }

        private void OpenConnection()
        {
            var connection = new ConnectionClass
                                 {
                                     Provider = Constants.ExOleDBDataSource,
                                     ConnectionString = _Url
            };

            connection.Open(_Url, null, null, 0);
            _Connection = connection;
        }


        public Connection(string baseUrl)
        {
            if (string.IsNullOrEmpty(baseUrl)) throw new ArgumentNullException("baseUrl");

            _Url = baseUrl;
            _ConnectionMustBeDisposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (_ConnectionMustBeDisposed && _Connection != null)
            {
                _Connection.Close();
                _Connection = null;
            }
        }

        ~Connection()
        {
            Dispose(false);
        }
    }
}
