﻿using System;
using System.Threading;
using ADODB;
using InfiniTec.Exchange.Eventing.Interop;

namespace InfiniTec.Exchange.Eventing
{
	public sealed class EventInfo : IDisposable
	{
		private bool _EventItemOpenedLocally;
		private Record _Item;
        public bool EventIsSynchronous { get; set; }
		internal bool IsSilentSaveOperation { get; set; }
		
		public string Url { get; internal set; }
		public EventFlags Flags { get; internal set; }

		public bool IsFolder { get; internal set;}
        public object UserState { get; set;}

		public Record Item
		{
			get
			{
				if (_Item == null)
				{
					OpenItemLocally();
				}
				return _Item;
			}
			internal set { _Item = value; }
		}

	    public string PermanentUrl
	    {
            get { return (string) Item.Fields[WellknownProperties.Item.PermanentUrl].Value; }
	    }

		private void OpenItemLocally()
		{
			_EventItemOpenedLocally = true;

			_Item = Connection.OpenItem(Url);
		}

		public Connection Connection { get; internal set; }
		public string SourceUrl { get; internal set; }
		public IExStoreDispEventInfo InternalEventInfo { get; internal set; }

        void IDisposable.Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			var item = Interlocked.Exchange(ref _Item, null);

			if (!_EventItemOpenedLocally || item == null)
			{
				return;
			}
			item.Close();
		}

		~EventInfo()
		{
			Dispose(false);
		}
	}
}