﻿using System.Diagnostics;

namespace InfiniTec.Exchange.Eventing
{
	public class Trace
	{
		private readonly TraceSource _Source;

		public Trace(string sourceName)
		{
			_Source = new TraceSource(sourceName);
		}

		public TraceOperation StartOperation(object operationId)
		{
			return new TraceOperation(this, operationId);
		}

		public TraceSource Source
		{
			get { return _Source; }
		}


		public void TraceData(TraceEventType eventType, int id, params object[] data)
		{
			Source.TraceData(eventType, id, data);
		}

		public void TraceData(TraceEventType eventType, int id, object data)
		{
			Source.TraceData(eventType, id, data);
		}

		public void TraceEvent(TraceEventType eventType, int id, string message)
		{
			Source.TraceEvent(eventType, id, message);
		}

		public void TraceError(string format, params object[] args)
		{
			TraceError(0, format, args);
		}

		public void TraceError(int id, string format, params object[] args)
		{
			TraceEvent(TraceEventType.Error, id, format, args);
		}

		public void TraceInformation(string format, params object[] args)
		{
			TraceInformation(0, format, args);
		}

		public void TraceInformation(int id, string format, params object[] args)
		{
			TraceEvent(TraceEventType.Information, id, format, args);
		}

		public void TraceWarning(string format, params object[] args)
		{
			TraceWarning(0, format, args);
		}

		public void TraceWarning(int id, string format, params object[] args)
		{
			TraceEvent(TraceEventType.Warning, id, format, args);
		}

		public void TraceVerbose(string format, params object[] args)
		{
			TraceVerbose(0, format, args);
		}

		public void TraceVerbose(int id, string format, params object[] args)
		{
			TraceEvent(TraceEventType.Verbose, id, format, args);
		}


		public void TraceEvent(TraceEventType eventType, int id, string format, params object[] args)
		{
			Source.TraceEvent(eventType, id, format, args);
		}
	}
}
